import os
import re

from collections import OrderedDict
from termcolor import colored

from addic7ed.shows import Shows
from addic7ed.config import Config

REGEX = r"(.*)\.[s|S]?([0-9]{1,2})[x|X|e|E]?([0-9]{2})\..*([0-9]+p)?.*"


class FileCrawler:
    def __init__(self):
        self.shows = Shows()
        self.episodes = OrderedDict()
        listfile = Config.paths or sorted(os.listdir())
        for f in listfile:
            if f.endswith((".avi", ".mkv", ".mp4")):
                ep = self._parse_filename(f)
                if ep:
                    self.episodes[f] = ep

    def _parse_filename(self, filename):
        m = re.match(REGEX, os.path.basename(filename))
        print(colored("%s... " % filename, "white", attrs=["dark"]),
              end="", flush=True)
        if m:
            serie = self.shows.get(m.group(1).replace('.', ' ').title())
            season = int(m.group(2))
            episode = int(m.group(3))
            print(colored("OK", "green"))

            return Episode(filename, serie, season, episode)
        else:
            print(colored("No match", "red"))
            return None


class Episode:
    def __init__(self, f, serie, season, episode):
        self.infos = {
            "serie": serie,
            "season": season,
            "episode": episode,
        }
        self.dir = os.path.dirname(f) or "."
        self.filename, self.ext = os.path.splitext(os.path.basename(f))
        m = re.search(r"-(.*)%s$" % self.ext, f)
        self.infos["group"] = m.group(1) if m else ""

    def rename(self, new_name):
        try:
            os.rename("%s/%s%s" % (self.dir, self.filename, self.ext),
                      "%s/%s%s" % (self.dir, new_name, self.ext))
            ret = colored("Renamed %s to %s" % (self.filename, new_name),
                          "green")
            self.filename = new_name
        except Exception as e:
            ret = colored(e, "red")

        return ret

    def __str__(self):
        return colored("%s - Season %02d Episode %02d (%s)" % (
            self.infos["serie"],
            self.infos["season"],
            self.infos["episode"],
            self.filename
        ), "blue")



import re
import requests

from bs4 import BeautifulSoup
from termcolor import colored
from fuzzywuzzy import fuzz

from addic7ed.constants import ADDIC7ED_URL
from addic7ed.config import Config


class Addic7edParser:
    def parse(self, serie, season, episode, group):
        url = "http://www.addic7ed.com/serie/%s/%s/%s" % (
            serie,
            season,
            episode
        )
        data = requests.get("%s/%s" % (url, Config.lang["code"])).text
        soup = BeautifulSoup(data, "html.parser")

        tables = soup.find_all("table", attrs={"class": "tabel95"})

        subs = []
        for table in tables:
            inner_table = table.find("table", attrs={"class": "tabel95"})
            if inner_table:
                subs.append(Subtitle(inner_table,
                                     "%s/%s" % (url, "addic7ed"),
                                     group))
        return sorted(subs,
                      key=lambda s: (s.match_ratio, s.downloads),
                      reverse=True)


class Subtitle:
    def __init__(self, html_table, referer, best_group):
        self.html = html_table
        self.referer = referer

        self.release = self._extract_release()
        self.match_ratio = fuzz.ratio(self.release.split(" ")[-1].upper(),
                                      best_group.upper())
        self.language = self._extract_language()
        self.completion = self._extract_completion()
        self.comment, self.various = self._extract_comment()
        m = re.search(r"([0-9]+) Downloads", self.various)
        self.downloads = int(m.group(1))
        self.link = self._extract_link()

    def download(self, dir, filename=None):
        if self.completion != "Completed":
            raise IncompleteError()

        subs = requests.get(self.link, headers={"Referer": self.referer})

        if not filename:
            filename = re.search(r'"(.*)"',
                                 subs.headers["Content-Disposition"]).group(1)
            filename = re.sub(r"\.%s.*Addic7ed\.com" %
                              re.escape(self.release.replace("Version ", "")),
                              "",  filename)
        if Config.keep_lang:
            f = open("%s/%s.%s%s" % (dir, filename[:-4],
                                     Config.lang["iso"],
                                     filename[-4:]), 'wb')
        else:
            f = open("%s/%s" % (dir, filename), 'wb')
        f.write(subs.content)
        f.close()
        return filename[:-4]

    def _extract_release(self):
        return self.html.find("td", attrs={"class": "NewsTitle"}) \
            .get_text(strip=True).split(",")[0]

    def _extract_language(self):
        return self.html.find("td", attrs={"class": "language"}) \
            .get_text(strip=True)

    def _extract_completion(self):
        return self.html.find("td", attrs={"width": "19%"}) \
            .get_text(strip=True)

    def _extract_comment(self):
        tds = self.html.find_all("td", attrs={"class": "newsDate"})
        return (tds[0].get_text(strip=True),
                tds[1].get_text(strip=True).replace("Â", ""))
        #  Damn, this replace() call sucks balls

    def _extract_link(self):
        return "%s%s" % (ADDIC7ED_URL,
                         self.html.find_all("a", attrs={
                             "class": "buttonDownload"
                         })[-1].get("href"))

    def __str__(self):
        complete = "green" if (self.completion == "Completed") else "red"
        return ("%s"  # " (%s)"
                "\n    %s | %s"
                "\n    %s"
                "\n    %s") % (
            colored(self.release, "white", attrs=['bold']),
            # colored("%02d%%" % self.match_ratio,
            #         "red" if self.match_ratio < 75 else "green"),
            self.language,
            colored(self.completion, complete),
            colored(self.comment, "white", attrs=["dark"]),
            colored(self.various, "white", attrs=["dark"])
        )


class IncompleteError(Exception):
    """ Raised when trying to download incomplete subtitle """
    def __str__(self):
        return "Can't download incomplete subtitle file..."



from termcolor import colored

from .parser import Addic7edParser
from .file_crawler import FileCrawler
from .logger import init_logger
from .config import Config


def addic7ed():
    try:
        init_logger()
        Config.load()
        main()
    except (EOFError, KeyboardInterrupt, SystemExit):
        print(colored("\nBye!", "yellow"))
        exit(0)


def main():
    crawler = FileCrawler()
    parser = Addic7edParser()
    print(colored("Found %d show(s)" % len(crawler.episodes), "green"),
          end="\n\n")

    for filename, ep in crawler.episodes.items():
        subs = parser.parse(**ep.infos)

        print(ep)

        if not subs:
            print(colored("No subtitles for %s" % filename, "red"), end="\n\n")
            continue

        for i, sub in enumerate(subs):
            print("[%s] %s" % (colored(i, "yellow"), sub))

        if Config.dry_run:
            print()
            continue
        else:
            version = input('Download number? ')
            if not version:
                print(colored("Nothing to do!", "yellow"),
                      end="\n\n")
                continue

            try:
                if Config.rename != "sub":
                    filename = subs[int(version)].download(ep.dir)
                    if filename and Config.rename == "video":
                        print(ep.rename(filename))
                else:
                    filename = subs[int(version)] \
                        .download(ep.dir, "%s.srt" % ep.filename)
                print(colored("Downloaded %s subtitle file" %
                              filename, "green"), end="\n\n")
            except Exception as e:
                print(colored(e, "red"),
                      end="\n\n")



ADDIC7ED_URL = "http://www.addic7ed.com"
CONFIG_FILE_NAME = ".addic7edrc"
LANG_DEFAULT = "en"

# Language code - "ISO 639-1"
LANG_ISO = {
    "all":      {"code": 0,     "name": "All languages"},
    "ar":       {"code": 38,    "name": "Arabic"},
    "az":       {"code": 48,    "name": "Azerbaijani"},
    "bg":       {"code": 35,    "name": "Bulgarian"},
    "bn":       {"code": 47,    "name": "Bengali"},
    "bs":       {"code": 44,    "name": "Bosnian"},
    "ca":       {"code": 12,    "name": "Català"},
    "cs":       {"code": 14,    "name": "Czech"},
    "da":       {"code": 30,    "name": "Danish"},
    "de":       {"code": 11,    "name": "German"},
    "el":       {"code": 27,    "name": "Greek"},
    "en":       {"code": 1,     "name": "English"},
    "es":       {"code": 4,     "name": "Spanish"},
    "es-la":    {"code": 6,     "name": "Spanish (Latin America)"},
    "es-es":    {"code": 5,     "name": "Spanish (Spain)"},
    "et":       {"code": 54,    "name": "Estonian"},
    "eu":       {"code": 13,    "name": "Euskera"},
    "fa":       {"code": 43,    "name": "Persian"},
    "fi":       {"code": 28,    "name": "Finnish"},
    "fr":       {"code": 8,     "name": "French"},
    "fr-ca":    {"code": 53,    "name": "French (Canadian)"},
    "gl":       {"code": 15,    "name": "Galego"},
    "he":       {"code": 23,    "name": "Hebrew"},
    "hi":       {"code": 55,    "name": "Hindi"},
    "hr":       {"code": 31,    "name": "Croatian"},
    "hu":       {"code": 20,    "name": "Hungarian"},
    "hy":       {"code": 50,    "name": "Armenian"},
    "id":       {"code": 37,    "name": "Indonesian"},
    "is":       {"code": 56,    "name": "Icelandic"},
    "it":       {"code": 7,     "name": "Italian"},
    "ja":       {"code": 32,    "name": "Japanese"},
    "ko":       {"code": 42,    "name": "Korean"},
    "lt":       {"code": 58,    "name": "Lithuanian"},
    "lv":       {"code": 57,    "name": "Latvian"},
    "mk":       {"code": 49,    "name": "Macedonian"},
    "ms":       {"code": 40,    "name": "Malay"},
    "nl":       {"code": 17,    "name": "Dutch"},
    "no":       {"code": 29,    "name": "Norwegian"},
    "pl":       {"code": 21,    "name": "Polish"},
    "pt":       {"code": 9,     "name": "Portuguese"},
    "pt-br":    {"code": 10,    "name": "Portuguese (Brazilian)"},
    "ro":       {"code": 26,    "name": "Romanian"},
    "ru":       {"code": 19,    "name": "Russian"},
    "si":       {"code": 60,    "name": "Sinhala"},
    "sk":       {"code": 25,    "name": "Slovak"},
    "sl":       {"code": 22,    "name": "Slovenian"},
    "sq":       {"code": 52,    "name": "Albanian"},
    "sr":       {"code": 39,    "name": "Serbian (Cyrillic)"},
    "sr-latn":  {"code": 39,    "name": "Serbian (Latin)"},
    "sv":       {"code": 18,    "name": "Swedish"},
    "ta":       {"code": 59,    "name": "Tamil"},
    "th":       {"code": 46,    "name": "Thai"},
    "tr":       {"code": 16,    "name": "Turkish"},
    "uk":       {"code": 51,    "name": "Ukrainian"},
    "vi":       {"code": 45,    "name": "Vietnamese"},
    "zh-hans":  {"code": 41,    "name": "Chinese (Simplified)"},
    "zh-hant":  {"code": 24,    "name": "Chinese (Traditional)"}
}



import requests

from bs4 import BeautifulSoup
from fuzzywuzzy import process
from termcolor import colored


class Shows:
    def __init__(self):
        print(colored("Fetching shows list, please wait...", "yellow"))
        data = requests.get("http://www.addic7ed.com/index.php").text
        soup = BeautifulSoup(data, "html.parser")
        self.list = [str(x.text) for x in
                     soup.find(id="qsShow").find_all("option")]

    def get(self, name):
        return process.extractOne(name, self.list)[0]



import logging
from os.path import expanduser, exists

from argparse import ArgumentParser, ArgumentTypeError, FileType
from configparser import ConfigParser

from .constants import LANG_ISO, LANG_DEFAULT, CONFIG_FILE_NAME

logger = logging.getLogger("addic7ed.config")
RENAME_MODES = ("none", "sub", "video")
DEFAULT = {
    "lang": LANG_DEFAULT,
    "rename": "none",
    "keep_lang": False,
}


def singleton(cls):
    instance = cls()

    def __call__():
        return instance

    return instance


def _valid_lang(lang):
    if lang not in LANG_ISO.keys():
        msg = "%s is not a valid language code." % lang
        raise ArgumentTypeError(msg)
    return lang


def _lang_list():
    llist = "%25s %s" % ("NAME", "CODE")
    for x in sorted(LANG_ISO.items(), key=lambda x: x[1]["name"]):
        llist = "%s\n%25s %s" % (llist, x[1]["name"], x[0])
    return llist


@singleton
class Config():
    def __init__(self):
        for k, v in DEFAULT.items():
            setattr(self, k, v)

    def load(self):
        self._from_file()
        self._from_args()

    def _from_file(self):
        logger.info("Loading config from file")
        config_file = ConfigParser()
        config_file.read("%s/%s" % (expanduser("~"), CONFIG_FILE_NAME))
        if "addic7ed" in config_file:
            for k, v in config_file["addic7ed"].items():
                if k in DEFAULT:
                    logger.info("Setting '%s' from config file: %s" % (k, v))
                    setattr(self, k, v)
                else:
                    logger.warn("Unknown key '%s' in config file" % k)

    def _from_args(self):
        logger.info("Loading config from CLI args")
        parser = ArgumentParser(description=(
            "Addic7ed scraper to download subtitles (almost) automatically"
        ))

        parser.add_argument("paths", metavar="PATH", type=str, nargs='*',
                            help=("path of file to search subtitles for ("
                                  "default: all video from current dir)."))
        parser.add_argument("--list-lang", action="store_true",
                            help="list supported languages.")
        parser.add_argument("-n", "--dry-run", action="store_true",
                            help=("do not ask or download subtitle"
                                  "just output available ones and leave."))
        parser.add_argument("-l", "--lang", type=_valid_lang,
                            help="language to search subs for (default: en).")
        parser.add_argument("-k", "--keep-lang", action="store_true",
                            help="suffix subtitle file with language code.")
        parser.add_argument("-f", "--from-file", type=FileType('r'),
                            help="read files paths from a file.")
        parser.add_argument("-r", "--rename",
                            choices=RENAME_MODES,
                            help=("rename sub/video to match video/sub "
                                  "or none at all (default: none)."))

        args = parser.parse_args()

        if args.list_lang:
            print(_lang_list())
            exit(0)

        self.dry_run = args.dry_run
        self.paths = [p for p in args.paths if exists(p)]

        if args.lang:
            logger.info("Setting 'lang' from config file: %s" % args.lang)
            self.lang = args.lang

        if args.from_file:
            files = sorted(args.from_file.read().splitlines())
            self.paths.extend([f for f in files if exists(f)])

        if args.keep_lang:
            self.keep_lang = args.keep_lang

        if args.rename:
            self.rename = args.rename

    def __setattr__(self, name, value):
        if name == "lang":
            if value in LANG_ISO.keys():
                logger.info("Language '%s' is now set." % value)
                super().__setattr__(name, dict({"iso": value},
                                               **LANG_ISO[value]))
            else:
                logger.warn("%s is not a valid language code, using %s" %
                            (value, self.lang["iso"]))
        elif name == "keep_lang":
            super().__setattr__(name, str(value) == "True")
        elif name == "rename":
            if value in RENAME_MODES:
                logger.info("Rename mode is now set to '%s'." % value)
                super().__setattr__(name, value)
            else:
                logger.warn("%s is not a valid rename mode, using %s" %
                            (value, self.rename))
        else:
            super().__setattr__(name, value)



from os import makedirs
from os.path import expanduser, exists

from logging import getLogger, Formatter, StreamHandler, DEBUG, WARN
from logging.handlers import RotatingFileHandler

from termcolor import colored

LOG_COLORS = {
    "DEBUG": "grey",
    "INFO": "cyan",
    "WARNING": "yellow",
    "ERROR": "magenta",
    "CRITICAL": "red"
}


def init_logger():
    logger = getLogger("addic7ed")
    logger.setLevel(DEBUG)

    directory = "%s/.config/addic7ed/" % expanduser("~")
    if not exists(directory):
        makedirs(directory)

    fh = RotatingFileHandler("%s%s" % (directory, "addic7ed.log"))
    fh.setLevel(DEBUG)

    sh = StreamHandler()
    sh.setLevel(WARN)

    fcolor = "%s - %s" % (colored("%(asctime)s", "green"),
                          "%(levelname)7s - %(name)s - %(message)s")
    formatter_color = ColoredFormatter(fcolor)
    formatter = Formatter(("%(asctime)s - %(levelname)7s - "
                           "%(name)s - %(message)s"))
    fh.setFormatter(formatter)
    sh.setFormatter(formatter_color)

    logger.addHandler(fh)
    logger.addHandler(sh)


class ColoredFormatter(Formatter):
    def format(self, record):
        record.msg = colored(record.msg, LOG_COLORS[record.levelname])
        return super().format(record)



from setuptools import setup, find_packages
from codecs import open

with open('README.rst', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='addic7ed',
    version='1.1.1',

    description=('Addic7ed scraper written in Python '
                 'to download subtitles (almost) automatically'),
    long_description=long_description,
    url='https://github.com/Jesus-21/addic7ed',
    author='Adrian MAURIN',
    author_email='adrian.maurin@gmail.com',
    license='MIT',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search'
    ],

    keywords='subtitles addic7ed scraper',
    packages=find_packages(exclude=['readme']),

    install_requires=['beautifulsoup4',
                      'fuzzywuzzy',
                      'python-Levenshtein',
                      'requests',
                      'termcolor'],

    entry_points={
        'console_scripts': [
            'addic7ed=addic7ed.__init__:addic7ed',
        ],
    },
)
